@@ -3,7 +3,8 @@ import re
 import sys
 
 from .bake import bake_schema, get_field_for
-from .metadata import decimal_metadata, metadata
+from .hooks import add_pre_load, pre_load
+from .metadata import datetime_metadata, decimal_metadata, metadata
 from .missing import MISSING
 from .naming_case import CAMEL_CASE, CAPITAL_CAMEL_CASE, DEFAULT_CASE, CamelCase, CapitalCamelCase, NamingCase
 from .options import NoneValueHandling, options
@@ -29,9 +30,12 @@ __all__: tuple[str, ...] = (
     "EmptySchema",
     "metadata",
     "decimal_metadata",
+    "datetime_metadata",
+    "pre_load",
+    "add_pre_load",
 )
 
-__version__ = "0.0.11"
+__version__ = "0.0.17"
 
 version = f"{__version__}, Python {sys.version}"
 
