@@ -5,7 +5,7 @@ import enum
 import inspect
 import types
 import uuid
-from typing import Any, Dict, Generic, List, Mapping, Type, TypeVar, cast
+from typing import Any, Dict, List, Mapping, Protocol, Type, TypeVar, cast
 
 import marshmallow as m
 import typing_inspect
@@ -25,6 +25,7 @@ from .fields import (
     str_field,
     uuid_field,
 )
+from .hooks import get_pre_loads
 from .naming_case import NamingCase
 from .options import NoneValueHandling, get_options_for
 
@@ -44,18 +45,27 @@ def bake_schema(
     if naming_case is None:
         naming_case = options.naming_case
 
-    fields = dataclasses.fields(cls)
+    fields_with_metadata = [
+        (field, _get_metadata(name=naming_case(field.name), default=_get_field_default(field), metadata=field.metadata))
+        for field in dataclasses.fields(cls)
+        if field.init
+    ]
+
+    for field, _ in fields_with_metadata:
+        for other_field, metadata in fields_with_metadata:
+            if field is other_field:
+                continue
+
+            other_field_name = metadata["name"]
+            if field.name == other_field_name:
+                raise ValueError(f"Invalid name={other_field_name} in metadata for field={other_field.name}")
+
     schema_class = type(
         cls.__name__,
         (_get_base_schema(cls, options.none_value_handling),),
         {
-            field.name: get_field_for(
-                field.type,
-                _get_metadata(name=naming_case(field.name), default=_get_field_default(field), metadata=field.metadata),
-                naming_case=naming_case,
-            )
-            for field in fields
-            if field.init
+            field.name: get_field_for(field.type, metadata, naming_case=naming_case)
+            for field, metadata in fields_with_metadata
         },
     )
     return cast(Type[m.Schema], schema_class)
@@ -72,34 +82,30 @@ def get_field_for(
 
     type = _substitute_any_to_open_generic(type)
 
-    if typing_inspect.is_union_type(type):
-        type_args = list(set(typing_inspect.get_args(type, True)))
-        if types.NoneType not in type_args or len(type_args) != 2:
-            raise ValueError(f"Unsupported {type=}")
+    if underlying_type_from_optional := _try_get_underlying_type_from_optional(type):
         required = False
-        type = next(type_arg for type_arg in type_args if type_arg is not types.NoneType)  # noqa
-    # to support new union syntax
-    elif isinstance(type, types.UnionType):
-        type_args = list(set(type.__args__))
-        if types.NoneType not in type_args or len(type_args) != 2:
-            raise ValueError(f"Unsupported {type=}")
+        allow_none = True
+        type = underlying_type_from_optional
+    elif metadata.get("default", dataclasses.MISSING) is not dataclasses.MISSING:
         required = False
-        type = next(type_arg for type_arg in type_args if type_arg is not types.NoneType)  # noqa
+        allow_none = False
     else:
         required = True
+        allow_none = False
 
     field_factory = _SIMPLE_TYPE_FIELD_FACTORIES.get(type)
     if field_factory:
-        typed_field_factory = cast(_FieldFactory[_T], field_factory)
-        return typed_field_factory(required=required, **metadata)
+        typed_field_factory = cast(_FieldFactory, field_factory)
+        return typed_field_factory(required=required, allow_none=allow_none, **metadata)
 
     if inspect.isclass(type) and issubclass(type, enum.Enum):
-        return enum_field(enum_type=type, required=required, **metadata)
+        return enum_field(enum_type=type, required=required, allow_none=allow_none, **metadata)
 
     if dataclasses.is_dataclass(type):
         return nested_field(
             bake_schema(type, naming_case=naming_case),
             required=required,
+            allow_none=allow_none,
             **metadata,
         )
 
@@ -109,11 +115,13 @@ def get_field_for(
             return list_field(
                 get_field_for(arguments[0], metadata={}, naming_case=naming_case),
                 required=required,
+                allow_none=allow_none,
                 **metadata,
             )
         if origin in (dict, Dict) and arguments[0] is str and arguments[1] is Any:
             return dict_field(
                 required=required,
+                allow_none=allow_none,
                 **metadata,
             )
 
@@ -137,6 +145,13 @@ if _MARSHMALLOW_VERSION_MAJOR >= 3:
             def post_load(self, data: dict[str, Any], **_: Any) -> Any:
                 return cls(**data)
 
+            @m.pre_load
+            def pre_load(self, data: dict[str, Any], **_: Any) -> Any:
+                result = data
+                for pre_load in get_pre_loads(cls):
+                    result = pre_load(result)
+                return result
+
         return _Schema
 
 else:
@@ -153,6 +168,15 @@ else:
             def post_load(self, data: dict[str, Any]) -> Any:
                 return cls(**data)
 
+            @m.pre_load  # type: ignore
+            def pre_load(self, data: dict[str, Any]) -> Any:
+                # Exclude unknown fields to prevent possible value overlapping
+                known_fields = {field.load_from or field.name for field in self.fields.values()}  # type: ignore
+                result = {key: value for key, value in data.items() if key in known_fields}
+                for pre_load in get_pre_loads(cls):
+                    result = pre_load(result)
+                return result
+
         return _Schema
 
 
@@ -163,11 +187,12 @@ def _get_field_default(field: dataclasses.Field[_T]) -> Any:
     return field.default
 
 
-class _FieldFactory(Generic[_T]):
+class _FieldFactory(Protocol):
     def __call__(
         self,
         *,
         required: bool,
+        allow_none: bool,
         name: str,
         default: Any,
         **kwargs: Any,
@@ -199,3 +224,20 @@ def _substitute_any_to_open_generic(type: type) -> type:
     if type is dict:
         return dict[Any, Any]
     return type
+
+
+def _try_get_underlying_type_from_optional(type: type) -> type | None:
+    if typing_inspect.is_union_type(type):
+        type_args = list(set(typing_inspect.get_args(type, True)))
+        if types.NoneType not in type_args or len(type_args) != 2:
+            raise ValueError(f"Unsupported {type=}")
+        return next(type_arg for type_arg in type_args if type_arg is not types.NoneType)  # noqa
+
+    # to support new union syntax
+    if isinstance(type, types.UnionType):
+        type_args = list(set(type.__args__))
+        if types.NoneType not in type_args or len(type_args) != 2:
+            raise ValueError(f"Unsupported {type=}")
+        return next(type_arg for type_arg in type_args if type_arg is not types.NoneType)  # noqa
+
+    return None
